@@ -27,12 +27,9 @@
     invisible()
 }
 
-.onLoad <- function(libname, pkgname) {
-  library.dynam("igraph", pkgname, libname, local=FALSE);
-  .Call("R_igraph_init", FALSE, FALSE, PACKAGE="igraph")
+.Call <- function(.NAME, ...) {
+  if (.NAME != "R_igraph_finalizer") {
+    base::.Call("R_igraph_check_finally_stack", PACKAGE="igraph")
+  }
+  base::.Call(.NAME, ...)
 }
-
-.onUnload <- function(libpath) {
-  library.dynam.unload("igraph", libpath)
-}
-
