@@ -1,3 +1,35 @@
+/* 
+ * Copyright 2007 Sandia Corporation. Under the terms of Contract
+ * DE-AC04-94AL85000 with Sandia Corporation, the U.S. Government retains
+ * certain rights in this software.
+ * 
+ * All rights reserved.
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions are 
+ * met:
+ * 
+ *     * Redistributions of source code must retain the above copyright 
+ * notice, this list of conditions and the following disclaimer.
+ *     * Redistributions in binary form must reproduce the above copyright 
+ * notice, this list of conditions and the following disclaimer in the 
+ * documentation and/or other materials provided with the distribution.
+ *     * Neither the name of Sandia National Laboratories nor the names of 
+ * its contributors may be used to endorse or promote products derived from 
+ * this software without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
+ * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
 // This file contains the member definitions of the DensityGrid.h class
 // This code is modified from the original code by B.N. Wylie
 
@@ -11,6 +43,7 @@ using namespace std;
 
 #include "drl_Node.h"
 #include "DensityGrid.h"
+#include "igraph_error.h"
 
 #define GET_BIN(y, x) (Bins[y*GRID_SIZE+x])
 
@@ -45,11 +78,12 @@ void DensityGrid::Init()
     }
   catch (bad_alloc errora)
     {
-      cout << "Error: Out of memory! Program stopped." << endl;
+      // cout << "Error: Out of memory! Program stopped." << endl;
 	  #ifdef MUSE_MPI
         MPI_Abort ( MPI_COMM_WORLD, 1 );
 	  #else
-	    exit (1);
+	    igraph_error("DrL is out of memory", __FILE__, __LINE__,
+			 IGRAPH_ENOMEM);
 	  #endif
     }
 	
@@ -187,12 +221,13 @@ void DensityGrid::Add(Node &N)
   if ( (x_grid >= GRID_SIZE) || (x_grid < 0) ||
        (y_grid >= GRID_SIZE) || (y_grid < 0) )
     {
-      cout << endl << "Error: Exceeded density grid with x_grid = " << x_grid 
-	       << " and y_grid = " << y_grid << ".  Program stopped." << endl;
+      // cout << endl << "Error: Exceeded density grid with x_grid = " << x_grid 
+      // 	       << " and y_grid = " << y_grid << ".  Program stopped." << endl;
       #ifdef MUSE_MPI
  	    MPI_Abort ( MPI_COMM_WORLD, 1 );
 	  #else
-	    exit (1);
+	    igraph_error("Exceeded density grid in DrL", __FILE__, 
+			 __LINE__, IGRAPH_EDRL);
 	  #endif
     }    
 
