@@ -1,7 +1,5 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph library.
+   igraph library.
    Copyright (C) 2007-2020 The igraph development team
 
    This program is free software; you can redistribute it and/or modify
@@ -28,19 +26,20 @@
 #include "igraph_interface.h"
 #include "igraph_memory.h"
 #include "igraph_qsort.h"
+#include "igraph_random.h"
 
 #include "core/interruption.h"
 
 /* Structure storing a community */
 typedef struct {
-    igraph_integer_t size;           /* Size of the community */
+    igraph_int_t size;           /* Size of the community */
     igraph_real_t weight_inside;     /* Sum of edge weights inside community */
     igraph_real_t weight_all;        /* Sum of edge weights starting/ending in the community */
 } igraph_i_multilevel_community;
 
 /* Global community list structure */
 typedef struct {
-    igraph_integer_t communities_no, vertices_no;  /* Number of communities, number of vertices */
+    igraph_int_t communities_no, vertices_no;  /* Number of communities, number of vertices */
     igraph_real_t weight_sum;              /* Sum of edges weight in the whole graph */
     igraph_i_multilevel_community *item;   /* List of communities */
     igraph_vector_int_t *membership;       /* Community IDs */
@@ -54,7 +53,7 @@ static igraph_real_t igraph_i_multilevel_community_modularity(
     igraph_real_t result = 0.0;
     igraph_real_t m = communities->weight_sum;
 
-    for (igraph_integer_t i = 0; i < communities->vertices_no; i++) {
+    for (igraph_int_t i = 0; i < communities->vertices_no; i++) {
         if (communities->item[i].size > 0) {
             result += (communities->item[i].weight_inside - resolution * communities->item[i].weight_all * communities->item[i].weight_all / m) / m;
         }
@@ -64,13 +63,13 @@ static igraph_real_t igraph_i_multilevel_community_modularity(
 }
 
 typedef struct {
-    igraph_integer_t from;
-    igraph_integer_t to;
-    igraph_integer_t id;
+    igraph_int_t from;
+    igraph_int_t to;
+    igraph_int_t id;
 } igraph_i_multilevel_link;
 
 static int igraph_i_multilevel_link_cmp(const void *a, const void *b) {
-    igraph_integer_t diff;
+    igraph_int_t diff;
 
     diff = ((igraph_i_multilevel_link*)a)->from - ((igraph_i_multilevel_link*)b)->from;
 
@@ -93,8 +92,8 @@ static int igraph_i_multilevel_link_cmp(const void *a, const void *b) {
 
 /* removes multiple edges and returns new edge IDs for each edge in |E|log|E| */
 static igraph_error_t igraph_i_multilevel_simplify_multiple(igraph_t *graph, igraph_vector_int_t *eids) {
-    igraph_integer_t ecount = igraph_ecount(graph);
-    igraph_integer_t l = -1, last_from = -1, last_to = -1;
+    igraph_int_t ecount = igraph_ecount(graph);
+    igraph_int_t l = -1, last_from = -1, last_to = -1;
     igraph_bool_t directed = igraph_is_directed(graph);
     igraph_vector_int_t edges;
     igraph_i_multilevel_link *links;
@@ -106,7 +105,7 @@ static igraph_error_t igraph_i_multilevel_simplify_multiple(igraph_t *graph, igr
     IGRAPH_CHECK_OOM(links, "Multi-level community structure detection failed.");
     IGRAPH_FINALLY(igraph_free, links);
 
-    for (igraph_integer_t i = 0; i < ecount; i++) {
+    for (igraph_int_t i = 0; i < ecount; i++) {
         links[i].from = IGRAPH_FROM(graph, i);
         links[i].to = IGRAPH_TO(graph, i);
         links[i].id = i;
@@ -116,7 +115,7 @@ static igraph_error_t igraph_i_multilevel_simplify_multiple(igraph_t *graph, igr
                  igraph_i_multilevel_link_cmp);
 
     IGRAPH_VECTOR_INT_INIT_FINALLY(&edges, 0);
-    for (igraph_integer_t i = 0; i < ecount; i++) {
+    for (igraph_int_t i = 0; i < ecount; i++) {
         if (links[i].from == last_from && links[i].to == last_to) {
             VECTOR(*eids)[links[i].id] = l;
             continue;
@@ -146,12 +145,12 @@ static igraph_error_t igraph_i_multilevel_simplify_multiple(igraph_t *graph, igr
 }
 
 typedef struct {
-    igraph_integer_t community;
+    igraph_int_t community;
     igraph_real_t weight;
 } igraph_i_multilevel_community_link;
 
 static int igraph_i_multilevel_community_link_cmp(const void *a, const void *b) {
-    igraph_integer_t diff = (
+    igraph_int_t diff = (
         ((igraph_i_multilevel_community_link*)a)->community -
         ((igraph_i_multilevel_community_link*)b)->community
     );
@@ -174,14 +173,14 @@ static int igraph_i_multilevel_community_link_cmp(const void *a, const void *b)
 static igraph_error_t igraph_i_multilevel_community_links(
         const igraph_t *graph,
         const igraph_i_multilevel_community_list *communities,
-        igraph_integer_t vertex, igraph_vector_int_t *edges,
+        igraph_int_t vertex, igraph_vector_int_t *edges,
         igraph_real_t *weight_all, igraph_real_t *weight_inside, igraph_real_t *weight_loop,
         igraph_vector_int_t *links_community, igraph_vector_t *links_weight) {
 
-    igraph_integer_t n, last = -1, c = -1;
+    igraph_int_t n, last = -1, c = -1;
     igraph_real_t weight = 1;
-    igraph_integer_t to, to_community;
-    igraph_integer_t community = VECTOR(*(communities->membership))[vertex];
+    igraph_int_t to, to_community;
+    igraph_int_t community = VECTOR(*(communities->membership))[vertex];
     igraph_i_multilevel_community_link *links;
 
     *weight_all = *weight_inside = *weight_loop = 0;
@@ -190,15 +189,15 @@ static igraph_error_t igraph_i_multilevel_community_links(
     igraph_vector_clear(links_weight);
 
     /* Get the list of incident edges */
-    IGRAPH_CHECK(igraph_incident(graph, edges, vertex, IGRAPH_ALL));
+    IGRAPH_CHECK(igraph_incident(graph, edges, vertex, IGRAPH_ALL, IGRAPH_LOOPS));
 
     n = igraph_vector_int_size(edges);
     links = IGRAPH_CALLOC(n, igraph_i_multilevel_community_link);
     IGRAPH_CHECK_OOM(links, "Multi-level community structure detection failed.");
     IGRAPH_FINALLY(igraph_free, links);
 
-    for (igraph_integer_t i = 0; i < n; i++) {
-        igraph_integer_t eidx = VECTOR(*edges)[i];
+    for (igraph_int_t i = 0; i < n; i++) {
+        igraph_int_t eidx = VECTOR(*edges)[i];
         weight = VECTOR(*communities->weights)[eidx];
 
         to = IGRAPH_OTHER(graph, eidx, vertex);
@@ -226,7 +225,7 @@ static igraph_error_t igraph_i_multilevel_community_links(
     /* Sort links by community ID and merge the same */
     igraph_qsort((void*)links, (size_t) n, sizeof(igraph_i_multilevel_community_link),
                  igraph_i_multilevel_community_link_cmp);
-    for (igraph_integer_t i = 0; i < n; i++) {
+    for (igraph_int_t i = 0; i < n; i++) {
         to_community = links[i].community;
         if (to_community != last) {
             IGRAPH_CHECK(igraph_vector_int_push_back(links_community, to_community));
@@ -246,7 +245,7 @@ static igraph_error_t igraph_i_multilevel_community_links(
 
 static igraph_real_t igraph_i_multilevel_community_modularity_gain(
         const igraph_i_multilevel_community_list *communities,
-        igraph_integer_t community, igraph_integer_t vertex,
+        igraph_int_t community, igraph_int_t vertex,
         igraph_real_t weight_all, igraph_real_t weight_inside,
         const igraph_real_t resolution) {
     IGRAPH_UNUSED(vertex);
@@ -262,8 +261,8 @@ static igraph_real_t igraph_i_multilevel_community_modularity_gain(
  * igraph_membership_reindex call */
 static igraph_error_t igraph_i_multilevel_shrink(igraph_t *graph, igraph_vector_int_t *membership) {
     igraph_vector_int_t edges;
-    igraph_integer_t no_of_nodes = igraph_vcount(graph);
-    igraph_integer_t no_of_edges = igraph_ecount(graph);
+    igraph_int_t no_of_nodes = igraph_vcount(graph);
+    igraph_int_t no_of_edges = igraph_ecount(graph);
     igraph_bool_t directed = igraph_is_directed(graph);
 
     IGRAPH_ASSERT(igraph_vector_int_size(membership) == no_of_nodes);
@@ -278,7 +277,7 @@ static igraph_error_t igraph_i_multilevel_shrink(igraph_t *graph, igraph_vector_
 
     /* Create the new edgelist */
     IGRAPH_CHECK(igraph_get_edgelist(graph, &edges, /* bycol= */ false));
-    for (igraph_integer_t i=0; i < 2*no_of_edges; i++) {
+    for (igraph_int_t i=0; i < 2*no_of_edges; i++) {
         VECTOR(edges)[i] = VECTOR(*membership)[ VECTOR(edges)[i] ];
     }
 
@@ -327,8 +326,8 @@ static igraph_error_t igraph_i_community_multilevel_step(
         igraph_real_t *modularity,
         const igraph_real_t resolution) {
 
-    igraph_integer_t vcount = igraph_vcount(graph);
-    igraph_integer_t ecount = igraph_ecount(graph);
+    igraph_int_t vcount = igraph_vcount(graph);
+    igraph_int_t ecount = igraph_ecount(graph);
     igraph_real_t q, pass_q;
     /* int pass; // used only for debugging */
     igraph_bool_t changed;
@@ -361,7 +360,7 @@ static igraph_error_t igraph_i_community_multilevel_step(
     IGRAPH_FINALLY(igraph_free, communities.item);
 
     /* Still initializing the communities data structure */
-    for (igraph_integer_t i = 0; i < vcount; i++) {
+    for (igraph_int_t i = 0; i < vcount; i++) {
         VECTOR(*communities.membership)[i] = i;
         communities.item[i].size = 1;
         communities.item[i].weight_inside = 0;
@@ -369,8 +368,8 @@ static igraph_error_t igraph_i_community_multilevel_step(
     }
 
     /* Some more initialization :) */
-    for (igraph_integer_t i = 0; i < ecount; i++) {
-        igraph_integer_t ffrom = IGRAPH_FROM(graph, i), fto = IGRAPH_TO(graph, i);
+    for (igraph_int_t i = 0; i < ecount; i++) {
+        igraph_int_t ffrom = IGRAPH_FROM(graph, i), fto = IGRAPH_TO(graph, i);
         igraph_real_t weight = 1;
 
         weight = VECTOR(*weights)[i];
@@ -385,7 +384,7 @@ static igraph_error_t igraph_i_community_multilevel_step(
     /* pass = 1; */
 
     do { /* Pass begin */
-        igraph_integer_t temp_communities_no = communities.communities_no;
+        igraph_int_t temp_communities_no = communities.communities_no;
 
         pass_q = q;
         changed = false;
@@ -393,14 +392,26 @@ static igraph_error_t igraph_i_community_multilevel_step(
         /* Save the current membership, it will be restored in case of worse result */
         IGRAPH_CHECK(igraph_vector_int_update(&temp_membership, communities.membership));
 
-        for (igraph_integer_t i = 0; i < vcount; i++) {
+        /* Apply a random inversion to the node_order permutation vector to help escape
+         * rare situations of an infinite loop. A full re-shuffling of node_order would
+         * have a measurable performance impact, hence the single inversion.
+         * See https://github.com/igraph/igraph/issues/2650 for details. */
+        if (vcount > 1) {
+            igraph_int_t i1 = RNG_INTEGER(0, vcount-1);
+            igraph_int_t i2 = RNG_INTEGER(0, vcount-1);
+            igraph_int_t tmp = VECTOR(node_order)[i1];
+            VECTOR(node_order)[i1] = VECTOR(node_order)[i2];
+            VECTOR(node_order)[i2] = tmp;
+        }
+
+        for (igraph_int_t i = 0; i < vcount; i++) {
             /* Exclude vertex from its current community */
             igraph_real_t weight_all = 0;
             igraph_real_t weight_inside = 0;
             igraph_real_t weight_loop = 0;
             igraph_real_t max_q_gain = 0;
             igraph_real_t max_weight;
-            igraph_integer_t old_id, new_id, n, ni;
+            igraph_int_t old_id, new_id, n, ni;
 
             ni = VECTOR(node_order)[i];
 
@@ -414,7 +425,7 @@ static igraph_error_t igraph_i_community_multilevel_step(
             new_id = old_id;
 
             /* Update old community */
-            igraph_vector_int_set(communities.membership, ni, -1);
+            VECTOR(*communities.membership)[ni] = -1;
             communities.item[old_id].size--;
             if (communities.item[old_id].size == 0) {
                 communities.communities_no--;
@@ -429,8 +440,8 @@ static igraph_error_t igraph_i_community_multilevel_step(
             max_weight = weight_inside;
             n = igraph_vector_int_size(&links_community);
 
-            for (igraph_integer_t j = 0; j < n; j++) {
-                igraph_integer_t c = VECTOR(links_community)[j];
+            for (igraph_int_t j = 0; j < n; j++) {
+                igraph_int_t c = VECTOR(links_community)[j];
                 igraph_real_t w = VECTOR(links_weight)[j];
 
                 igraph_real_t q_gain =
@@ -447,7 +458,7 @@ static igraph_error_t igraph_i_community_multilevel_step(
             /* debug("Added vertex %ld to community %ld (gain %lf).\n", ni, new_id, (double) max_q_gain); */
 
             /* Add vertex to "new" community and update it */
-            igraph_vector_int_set(communities.membership, ni, new_id);
+            VECTOR(*communities.membership)[ni] = new_id;
             if (communities.item[new_id].size == 0) {
                 communities.communities_no++;
             }
@@ -501,9 +512,9 @@ static igraph_error_t igraph_i_community_multilevel_step(
 
     /* We reuse the links_weight vector to store the old edge weights */
     IGRAPH_CHECK(igraph_vector_update(&links_weight, weights));
-    igraph_vector_fill(weights, 0);
+    igraph_vector_null(weights);
 
-    for (igraph_integer_t i = 0; i < ecount; i++) {
+    for (igraph_int_t i = 0; i < ecount; i++) {
         VECTOR(*weights)[VECTOR(edges)[i]] += VECTOR(links_weight)[i];
     }
 
@@ -520,16 +531,10 @@ static igraph_error_t igraph_i_community_multilevel_step(
 /**
  * \ingroup communities
  * \function igraph_community_multilevel
- * \brief Finding community structure by multi-level optimization of modularity.
+ * \brief Finding community structure by multi-level optimization of modularity (Louvain).
  *
- * This function implements the multi-level modularity optimization
- * algorithm for finding community structure, see
- * Blondel, V. D., Guillaume, J.-L., Lambiotte, R., &amp; Lefebvre, E. (2008). Fast
- * unfolding of communities in large networks. Journal of Statistical Mechanics:
- * Theory and Experiment, 10008(10), 6.
- * https://doi.org/10.1088/1742-5468/2008/10/P10008 for the details (preprint:
- * http://arxiv.org/abs/0803.0476). The algorithm is sometimes known as the
- * "Louvain" algorithm.
+ * This function implements a multi-level modularity optimization algorithm
+ * for finding community structure, sometimes known as the Louvain algorithm.
  *
  * </para><para>
  * The algorithm is based on the modularity measure and a hierarchical approach.
@@ -542,16 +547,25 @@ static igraph_error_t igraph_i_community_multilevel_step(
  * the modularity cannot be increased any more in a step.
  *
  * </para><para>
- * The resolution parameter \c gamma allows finding communities at different
+ * The resolution parameter \c γ allows finding communities at different
  * resolutions. Higher values of the resolution parameter typically result in
  * more, smaller communities. Lower values typically result in fewer, larger
  * communities. The original definition of modularity is retrieved when setting
- * <code>gamma=1</code>. Note that the returned modularity value is calculated using
+ * <code>γ=1</code>. Note that the returned modularity value is calculated using
  * the indicated resolution parameter. See \ref igraph_modularity() for more details.
  *
  * </para><para>
  * The original version of this function was contributed by Tom Gregorovic.
  *
+ * </para><para>
+ * Reference:
+ *
+ * </para><para>
+ * Blondel, V. D., Guillaume, J.-L., Lambiotte, R., &amp; Lefebvre, E.:
+ * Fast unfolding of communities in large networks.
+ * Journal of Statistical Mechanics: Theory and Experiment, 10008(10), 6 (2008).
+ * https://doi.org/10.1088/1742-5468/2008/10/P10008
+ *
  * \param graph       The input graph. It must be an undirected graph.
  * \param weights     Numeric vector containing edge weights. If \c NULL, every edge
  *                    has equal weight. The weights are expected to be non-negative.
@@ -587,9 +601,9 @@ igraph_error_t igraph_community_multilevel(const igraph_t *graph,
     igraph_vector_int_t m;
     igraph_vector_int_t level_membership;
     igraph_real_t prev_q = -1, q = -1;
-    igraph_integer_t level = 1;
-    igraph_integer_t vcount = igraph_vcount(graph);
-    igraph_integer_t ecount = igraph_ecount(graph);
+    igraph_int_t level = 1;
+    igraph_int_t vcount = igraph_vcount(graph);
+    igraph_int_t ecount = igraph_ecount(graph);
 
     /* Initial sanity checks on the input parameters */
     if (igraph_is_directed(graph)) {
@@ -630,7 +644,7 @@ igraph_error_t igraph_community_multilevel(const igraph_t *graph,
 
     if (memberships || membership) {
         /* Put each vertex in its own community */
-        for (igraph_integer_t i = 0; i < vcount; i++) {
+        for (igraph_int_t i = 0; i < vcount; i++) {
             VECTOR(level_membership)[i] = i;
         }
     }
@@ -645,7 +659,7 @@ igraph_error_t igraph_community_multilevel(const igraph_t *graph,
 
     while (true) {
         /* Remember the previous modularity and vertex count, do a single step */
-        igraph_integer_t step_vcount = igraph_vcount(&g);
+        igraph_int_t step_vcount = igraph_vcount(&g);
 
         prev_q = q;
         IGRAPH_CHECK(igraph_i_community_multilevel_step(&g, &w, &m, &q, resolution));
@@ -656,7 +670,7 @@ igraph_error_t igraph_community_multilevel(const igraph_t *graph,
         }
 
         if (memberships || membership) {
-            for (igraph_integer_t i = 0; i < vcount; i++) {
+            for (igraph_int_t i = 0; i < vcount; i++) {
                 /* Readjust the membership vector */
                 VECTOR(level_membership)[i] = VECTOR(m)[ VECTOR(level_membership)[i] ];
             }
@@ -703,7 +717,7 @@ igraph_error_t igraph_community_multilevel(const igraph_t *graph,
     /* If we need the final membership vector, copy it to the output */
     if (membership) {
         IGRAPH_CHECK(igraph_vector_int_resize(membership, vcount));
-        for (igraph_integer_t i = 0; i < vcount; i++) {
+        for (igraph_int_t i = 0; i < vcount; i++) {
             VECTOR(*membership)[i] = VECTOR(level_membership)[i];
         }
     }
