@@ -1,7 +1,27 @@
+/*
+ *
+ * gengraph - generation of random simple connected graphs with prescribed
+ *            degree sequence
+ *
+ * Copyright (C) 2006  Fabien Viger
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
 #include "gengraph_definitions.h"
-#include <assert.h>
-#include <stdio.h>
-#include <math.h>
+#include <cassert>
+#include <cstdio>
+#include <cmath>
 #include <limits>
 
 #include "gengraph_qsort.h"
@@ -10,6 +30,14 @@
 #include "gengraph_degree_sequence.h"
 #include "gengraph_graph_molloy_optimized.h"
 
+#include "igraph_error.h"
+#include "igraph_statusbar.h"
+#include "igraph_progress.h"
+
+#ifndef register
+#define register
+#endif
+
 using namespace std;
 
 namespace gengraph {
@@ -64,54 +92,54 @@ graph_molloy_opt::graph_molloy_opt(degree_sequence &degs) {
   alloc(degs);
 }
 
-graph_molloy_opt::graph_molloy_opt(FILE *f) { 
-  char *buff = new char[FBUFF_SIZE];
-  // How many vertices ?
-  if(VERBOSE()) fprintf(stderr,"Read file: #vertices=");
-  int i;
-  int n=0;
-  while(fgets(buff,FBUFF_SIZE,f)) if(sscanf(buff,"%d",&i)==1 && i>n) n=i;
-  n++;
-  // degrees ?
-  if(VERBOSE()) fprintf(stderr,"%d, #edges=",n);
-  int *degs = new int[n];
-  for(i=0; i<n; i++) degs[i]=0;
-  rewind(f);
-  while(fgets(buff,FBUFF_SIZE,f)) {
-    int d = 0;
-    if(sscanf(buff,"%d",&i)==1) {
-      char *b = buff;
-      while(skip_int(b)) d++;
-      degs[i]=d;
-    }
-  }
-  // allocate memory
-  degree_sequence dd(n,degs);
-  a = dd.sum();
-  if(VERBOSE()) fprintf(stderr,"%d\nAllocating memory...",a);
-  alloc(dd);
-  // add edges
-  if(VERBOSE()) fprintf(stderr,"done\nCreating edges...");
-  rewind(f);
-  int line=0;
-  int j;
-  while(fgets(buff,FBUFF_SIZE,f)) {
-    line++;
-    if(sscanf(buff,"%d",&i)==1) {
-      char *b = buff;
-      while(skip_int(b)) {
-        if(sscanf(b,"%d",&j)!=1) {
-          fprintf(stderr,"\nParse error at line %d, col %d : integer expected\n",line,int(b-buff));
-          exit(6);
-        }
-        *(neigh[i]++) = j;
-      }
-    }
-  }
-  delete[] buff;
-  compute_neigh();
-  if(VERBOSE()) fprintf(stderr,"done\n");
-}
+// graph_molloy_opt::graph_molloy_opt(FILE *f) { 
+//   char *buff = new char[FBUFF_SIZE];
+//   // How many vertices ?
+//   if(VERBOSE()) fprintf(stderr,"Read file: #vertices=");
+//   int i;
+//   int n=0;
+//   while(fgets(buff,FBUFF_SIZE,f)) if(sscanf(buff,"%d",&i)==1 && i>n) n=i;
+//   n++;
+//   // degrees ?
+//   if(VERBOSE()) fprintf(stderr,"%d, #edges=",n);
+//   int *degs = new int[n];
+//   for(i=0; i<n; i++) degs[i]=0;
+//   rewind(f);
+//   while(fgets(buff,FBUFF_SIZE,f)) {
+//     int d = 0;
+//     if(sscanf(buff,"%d",&i)==1) {
+//       char *b = buff;
+//       while(skip_int(b)) d++;
+//       degs[i]=d;
+//     }
+//   }
+//   // allocate memory
+//   degree_sequence dd(n,degs);
+//   a = dd.sum();
+//   if(VERBOSE()) fprintf(stderr,"%d\nAllocating memory...",a);
+//   alloc(dd);
+//   // add edges
+//   if(VERBOSE()) fprintf(stderr,"done\nCreating edges...");
+//   rewind(f);
+//   int line=0;
+//   int j;
+//   while(fgets(buff,FBUFF_SIZE,f)) {
+//     line++;
+//     if(sscanf(buff,"%d",&i)==1) {
+//       char *b = buff;
+//       while(skip_int(b)) {
+//         if(sscanf(b,"%d",&j)!=1) {
+//           fprintf(stderr,"\nParse error at line %d, col %d : integer expected\n",line,int(b-buff));
+//           exit(6);
+//         }
+//         *(neigh[i]++) = j;
+//       }
+//     }
+//   }
+//   delete[] buff;
+//   compute_neigh();
+//   if(VERBOSE()) fprintf(stderr,"done\n");
+// }
 
 graph_molloy_opt::graph_molloy_opt(int *svg) {
   // Read n
@@ -258,7 +286,7 @@ int* graph_molloy_opt::components(int *comp) {
   for(i=nb_comp-1; i>=0; i--) buff[i] = --box[buff[i]-offset];
   delete[] box;
   // reassign component indexes
-  for(int *c=comp+n; comp!=c--; *c=buff[*c-1]);
+  for(int *c=comp+n; comp!=c--; *c=buff[*c-1]) { }
   // clean.. at last!
   delete[] buff;
   return comp;
@@ -344,13 +372,13 @@ bool graph_molloy_opt::havelhakimi() {
       dc--;
     }
     if(dv != 0) { // We couldn't bind entirely v
-      if(VERBOSE()) {
-        fprintf(stderr,"Error in graph_molloy_opt::havelhakimi() :\n");
-        fprintf(stderr,"Couldn't bind vertex %d entirely (%d edges remaining)\n",v,dv);
-      }
       delete[] nb;
       delete[] sorted;
       compute_neigh();
+      igraph_errorf("Error in graph_molloy_opt::havelhakimi():"
+		    " Couldn't bind vertex %d entirely "
+		    "(%d edges remaining)", __FILE__, __LINE__, 
+		    IGRAPH_EINTERNAL, v, dv);
       return false;
     }
   }
@@ -363,7 +391,7 @@ bool graph_molloy_opt::havelhakimi() {
 
 bool graph_molloy_opt::is_connected() {
   bool *visited = new bool[n];
-  for(int i=n; i>0; visited[--i]=false);
+  for(int i=n; i>0; visited[--i]=false) { }
   int *to_visit = new int[n];
   int *stop = to_visit;
   int left = n-1;
@@ -388,7 +416,7 @@ bool graph_molloy_opt::is_connected() {
 bool graph_molloy_opt::make_connected() {
   //assert(verify());
   if(a/2 < n-1) {
-    fprintf(stderr,"\ngraph::make_connected() failed : #edges < #vertices-1\n");
+    // fprintf(stderr,"\ngraph::make_connected() failed : #edges < #vertices-1\n");
     return false;
   }
   int i;
@@ -401,7 +429,7 @@ bool graph_molloy_opt::make_connected() {
   unsigned char * dist  = new unsigned char[n];
 #define NOT_VISITED 255
 #define FORBIDDEN   254
-  for(i=n; i>0; dist[--i]=NOT_VISITED);
+  for(i=n; i>0; dist[--i]=NOT_VISITED) { }
 
 // Data struct to store components : either surplus trees or surplus edges are stored at buff[]'s end
 // - A Tree is coded by one of its vertices
@@ -419,9 +447,11 @@ bool graph_molloy_opt::make_connected() {
   for(int v0=0; v0<n; v0++) if(dist[v0]==NOT_VISITED) {
     // is v0 an isolated vertex?
     if(deg[v0]==0) {
-      if(VERBOSE()) fprintf(stderr,"graph_molloy_opt::make_connected() returned FALSE : vertex %d has degree 0\n",v0);
       delete[] dist;
       delete[] buff;
+      igraph_errorf("graph_molloy_opt::make_connected() returned FALSE : "
+		    "vertex %d has degree 0", __FILE__, __LINE__, 
+		    IGRAPH_EINTERNAL, v0);
       return false;
     }
     dist[v0] = 0; // root
@@ -841,7 +871,7 @@ void graph_molloy_opt::add_traceroute_edge(int v, int k, int *newdeg, double **e
   }
   // if edge redudancy is asked, look for dual edge
   else if(edge_redudancy!=NULL)
-    for(int *ww = neigh[w]; *(ww++)!=v; k2++);
+    for(int *ww = neigh[w]; *(ww++)!=v; k2++) { }
   // add edge redudancy
   if(edge_redudancy!=NULL) {
     edge_redudancy[v][k]  += red;
@@ -878,8 +908,8 @@ int graph_molloy_opt::breadth_path_search(int src, int *buff, double *paths, uns
         if(++nb_visited==n) last_dist=nd;
       }
       else if(d==nd) if((paths[w]+=p)==numeric_limits<double>::infinity()) {
-        fprintf(stderr,"Fatal error : too many (>MAX_DOUBLE) possible paths in graph\n");
-        exit(-1);
+	  IGRAPH_ERROR("Fatal error : too many (>MAX_DOUBLE) possible"
+		       " paths in graph", IGRAPH_EOVERFLOW);
       }
     }
   }
@@ -901,7 +931,7 @@ void graph_molloy_opt::explore_usp(double *target, int nb_vertices, int *buff, d
       double f = 0.0;
       int father = -1;
       while(f<father_index) {
-        while(dist[father=ww[k++]]!=pd);
+        while(dist[father=ww[k++]]!=pd) { }
         f+=paths[father];
       }
       // increase target[] of father
@@ -973,7 +1003,7 @@ void graph_molloy_opt::explore_rsp(double *target, int nb_vertices, int *buff, d
 
 double *graph_molloy_opt::vertex_betweenness(int mode, bool trivial_paths) {
   char MODES[3] = {'U','A','R'};
-  if(VERBOSE()) fprintf(stderr,"Computing vertex betweenness %cSP...",MODES[mode]);
+  igraph_statusf("Computing vertex betweenness %cSP...", 0, MODES[mode]);
   
   // breadth-first search vertex fifo
   int *buff = new int[n];
@@ -988,16 +1018,18 @@ double *graph_molloy_opt::vertex_betweenness(int mode, bool trivial_paths) {
   // init all
   int progress = 0;
   memset(dist,0,sizeof(unsigned char)*n); 
-  for(double *yo = target+n; (yo--)!=target; *yo=1.0);
-  for(double *yo = b+n; (yo--)!=b; *yo=0.0);
+  for(double *yo = target+n; (yo--)!=target; *yo=1.0) { }
+  for(double *yo = b+n; (yo--)!=b; *yo=0.0) { }
   
   int progress_steps = max(1000,n/10);
   // Main loop
   for(int v0 = 0; v0<n; v0++) {
     // Verbose
-    if(VERBOSE()==VERBOSE_LOTS && v0>(progress*n) / progress_steps) {
+    if(v0>(progress*n) / progress_steps) {
       progress++;
-      fprintf(stderr,"\rComputing vertex betweenness %cSP : %3f%%",MODES[mode], 100.0*double(progress)/double(progress_steps));
+      igraph_progressf("Computing vertex betweenness %cSP",
+		       100.0*double(progress)/double(progress_steps), 0,
+		       MODES[mode]);
     }
     // Breadth-first search
     int nb_vertices = breadth_path_search(v0, buff, paths, dist);
@@ -1012,26 +1044,27 @@ double *graph_molloy_opt::vertex_betweenness(int mode, bool trivial_paths) {
       case MODE_RSP:
         explore_rsp(target, nb_vertices, buff, paths, dist); break;
       default:
-        fprintf(stderr,"Warning : graph_molloy_opt::vertex_betweenness() called with Invalid Mode\n");
+        IGRAPH_WARNING("graph_molloy_opt::vertex_betweenness() "
+		       "called with Invalid Mode");
     }
     // add targets[vertices in component] to global betweenness and reset targets[]
     if(nb_vertices==n) {
       // cache optimization if all vertices are in component
       double *bb=b;
       double *tt_end=target+n;
-      if(trivial_paths) for(double *yo=target; yo!=tt_end; *(bb++)+=*(yo++));
+      if(trivial_paths) for(double *yo=target; yo!=tt_end; *(bb++)+=*(yo++)){}
       else { 
-	for(double *yo=target; yo!=tt_end; *(bb++)+=(*(yo++)-1.0));
+	for(double *yo=target; yo!=tt_end; *(bb++)+=(*(yo++)-1.0)) { }
 	b[*buff]-=(target[*buff]-1.0);
       }
-      for(double *yo = target; yo!=tt_end; *(yo++)=1.0);
+      for(double *yo = target; yo!=tt_end; *(yo++)=1.0) { }
     }
     else {
       if(trivial_paths) 
-        for(int *yo = buff+nb_vertices; (yo--)!=buff; b[*yo]+=target[*yo]);
+        for(int *yo = buff+nb_vertices; (yo--)!=buff; b[*yo]+=target[*yo]) { }
       else
-        for(int *yo = buff+nb_vertices; (--yo)!=buff; b[*yo]+=(target[*yo]-1.0));
-      for(int *yo = buff+nb_vertices; (yo--)!=buff; target[*yo]=1.0);
+        for(int *yo = buff+nb_vertices; (--yo)!=buff; b[*yo]+=(target[*yo]-1.0)) { }
+      for(int *yo = buff+nb_vertices; (yo--)!=buff; target[*yo]=1.0) { }
     }
   }
   // Clean all & return
@@ -1039,7 +1072,7 @@ double *graph_molloy_opt::vertex_betweenness(int mode, bool trivial_paths) {
   delete[] dist;
   delete[] buff;
   delete[] paths;
-  if(VERBOSE()) fprintf(stderr,"Done\n");
+  igraph_status("Done\n", 0);
   return b;
 }
 
@@ -1047,8 +1080,8 @@ double graph_molloy_opt::traceroute_sample(int mode, int nb_src, int *src, int n
   // verify & verbose
   assert(verify());
   char MODES[3] = {'U','A','R'};
-  if(VERBOSE()==VERBOSE_LOTS)
-    fprintf(stderr,"traceroute %cSP on G(N=%d,M=%d) with %d src and %d dst...",MODES[mode],nbvertices_real(),nbarcs(),nb_src,nb_dst);
+  igraph_statusf("traceroute %cSP on G(N=%d,M=%d) with %d src and %d dst...",
+		 0, MODES[mode], nbvertices_real(), nbarcs(), nb_src,nb_dst);
 
   // create dst[] buffer if necessary
   bool newdist = dst==NULL;
@@ -1068,9 +1101,9 @@ double graph_molloy_opt::traceroute_sample(int mode, int nb_src, int *src, int n
   int i;
   memset(dist,0,sizeof(unsigned char)*n); 
   memset(newdeg,0,sizeof(int)*n); 
-  for(double *yo = target+n; (yo--)!=target; *yo=0.0);
+  for(double *yo = target+n; (yo--)!=target; *yo=0.0) { }
   if(redudancy!=NULL)
-    for(double *yo = redudancy+n; (yo--)!=redudancy; *yo=0.0);
+    for(double *yo = redudancy+n; (yo--)!=redudancy; *yo=0.0) { } 
 
   // src_0 counts the number of sources having degree 0
   int src_0 = 0;
@@ -1117,12 +1150,13 @@ double graph_molloy_opt::traceroute_sample(int mode, int nb_src, int *src, int n
       case MODE_RSP:
         explore_rsp(target, nb_vertices, buff, paths, dist, newdeg, edge_redudancy); break;
       default:
-        fprintf(stderr,"Warning : graph_molloy_opt::traceroute_sample() called with Invalid Mode\n");
+        IGRAPH_WARNING("graph_molloy_opt::traceroute_sample() called "
+		       "with Invalid Mode");
     }
     // add target[] to redudancy[] if needed
     if(redudancy!=NULL) for(i=1; i<nb_vertices; i++) redudancy[buff[i]]+=(target[buff[i]]);
     // clear target[]
-    for(int *yo = buff+nb_vertices; yo-- != buff; target[*yo]=0.0);
+    for(int *yo = buff+nb_vertices; yo-- != buff; target[*yo]=0.0) { }
   }
   // update degrees
   for(i=0; i<n; i++) deg[i]=newdeg[i];
@@ -1134,10 +1168,13 @@ double graph_molloy_opt::traceroute_sample(int mode, int nb_src, int *src, int n
   delete[] newdeg;
   delete[] target;
   if(newdist) delete[] dst;
-  if(VERBOSE()) {
-    if(VERBOSE()==VERBOSE_LOTS) fprintf(stderr,"discovered %d vertices and %d edges\n",nbvertices_real(),nbarcs());
-    if(src_0)  fprintf(stderr,"Warning : %d sources had degree 0\n",src_0);
-    if(nopath) fprintf(stderr,"Warning : %d (src,dst) pairs had no possible path\n",nopath);
+  {
+    igraph_statusf("discovered %d vertices and %d edges\n", 0, 
+		   nbvertices_real(), nbarcs());
+    if(src_0)  igraph_warningf("%d sources had degree 0\n", __FILE__, 
+			       __LINE__, -1, src_0);
+    if(nopath) igraph_warningf("%d (src,dst) pairs had no possible path\n",
+			       __FILE__, __LINE__, -1, nopath);
   }
   return total_dist/double(nb_paths);
 }  
@@ -1172,8 +1209,8 @@ double graph_molloy_opt::path_sampling(int *nb_dst, int *dst, double* redudancie
   if(NOMEM) dst = new int[n];
   int i;
   int next_step = n+1;
-  if(VERBOSE()) {
-    fprintf(stderr,"Sampling paths");
+  {
+    igraph_status("Sampling paths", 0);
     next_step = 0;
   }
   // breadth-first search buffers buff[] and dist[]
@@ -1201,8 +1238,7 @@ double graph_molloy_opt::path_sampling(int *nb_dst, int *dst, double* redudancie
     if(deg[s]==0) src_0++; else {
       if(s>next_step) {
         next_step = s+(n/1000)+1;
-        fprintf(stderr,"\rSampling paths : ");
-        print_percent(double(s)/double(n));
+        igraph_progress("Sampling paths", double(s)/double(n), 0);
       }
       int v;
       // breadth-first search
@@ -1231,7 +1267,7 @@ double graph_molloy_opt::path_sampling(int *nb_dst, int *dst, double* redudancie
       if(NOMEM) pick_random_src(double(t_index),NULL,dst);
       while(t_index--) if(dist[v = *(dst++)]==0) nopath++; else {
 #ifdef _DEBUG
-        if(VERBOSE()==VERBOSE_LOTS) fprintf(stderr,"Sampling path %d -> %d\n",s,v);
+	  igraph_statusf("Sampling path %d -> %d\n", 0, s, v);
 #endif //_DEBUG
         nb_paths++;
         // while we haven't reached the source..
@@ -1274,9 +1310,11 @@ double graph_molloy_opt::path_sampling(int *nb_dst, int *dst, double* redudancie
   delete[] nb_pos;
   if(NOMEM) delete[] dst;
   if(VERBOSE()) {
-    fprintf(stderr,"\rSampling paths :  Done   \n");
-    if(src_0)  fprintf(stderr,"Warning : %d sources had degree 0\n",src_0);
-    if(nopath) fprintf(stderr,"Warning : %d (src,dst) pairs had no possible path\n",nopath);
+    igraph_status("Sampling paths :  Done   \n", 0);
+    if(src_0)  igraph_warningf("%d sources had degree 0", __FILE__, 
+			       __LINE__, -1, src_0);
+    if(nopath) igraph_warningf("%d (src,dst) pairs had no possible path",
+			       __FILE__, __LINE__, -1, nopath);
   }
   double tdist = double(total_dist64);
   if(total_dist64>0) tdist *= 4294967296.0;
@@ -1291,14 +1329,15 @@ int *graph_molloy_opt::vertices_real(int &nb_v) {
     for(yo=deg; yo!=deg+n; ) if(*(yo++)>0) nb_v++;
   }
   if(nb_v==0) {
-    if(VERBOSE()) fprintf(stderr,"Waring: graph is empty\n");
+    IGRAPH_WARNING("graph is empty");
     return NULL;
   }
   int *buff=new int[nb_v];
   yo=buff;
   for(int i=0; i<n; i++) if(deg[i]>0) *(yo++)=i;
   if(yo!=buff+nb_v){
-    fprintf(stderr,"Warning: wrong #vertices in graph_molloy_opt::vertices_real(%d)\n",nb_v);
+    igraph_warningf("wrong #vertices in graph_molloy_opt::vertices_real(%d)",
+		    __FILE__, __LINE__, -1, nb_v);
     delete[] buff;
     return NULL;
   }
@@ -1313,7 +1352,8 @@ int *graph_molloy_opt::pick_random_vertices(int &k, int *output, int nb_v, int *
     CREATED_AMONG=true;
   }
   if(k>nb_v) {
-    fprintf(stderr,"Warning : tried to pick %d among %d vertices. Picked only %d\n",k,nb_v,nb_v);
+    igraph_warningf("Warning : tried to pick %d among %d vertices. "
+		    "Picked only %d", __FILE__, __LINE__, -1, k, nb_v, nb_v);
     k = nb_v;
   }
   if(k>0) {
@@ -1372,7 +1412,7 @@ int graph_molloy_opt::core() {
 
 int graph_molloy_opt::try_disconnect(int K, int max_tries) {
   bool *visited = new bool[n];
-  for(bool *p = visited+n; p!=visited; *(--p)=false);
+  for(bool *p = visited+n; p!=visited; *(--p)=false) { }
   int *Kbuff = new int[K];
   int tries = 0;
   int next_step = -1;
@@ -1380,7 +1420,8 @@ int graph_molloy_opt::try_disconnect(int K, int max_tries) {
   bool yo = true;
   while(yo && tries<max_tries) {
     if(tries==next_step) {
-      fprintf(stderr,"\rTrying to disconnect the graph... %d edges swaps done so far",tries);
+      igraph_statusf("Trying to disconnect the graph... "
+		       "%d edges swaps done so far", 0, tries);
       next_step += 100;
     }
     int v1 = pick_random_vertex();
@@ -1453,7 +1494,7 @@ double graph_molloy_opt::rho(int mode, int nb_src, int *src, int nb_dst, int *ds
   int i;
   memset(dist,0,sizeof(unsigned char)*n); 
   memset(times_seen,0,sizeof(int)*n); 
-  for(double *yo = target+n; (yo--)!=target; *yo=0.0);
+  for(double *yo = target+n; (yo--)!=target; *yo=0.0) { }
 
   // src_0 counts the number of sources having degree 0
   int src_0 = 0;
@@ -1481,7 +1522,7 @@ double graph_molloy_opt::rho(int mode, int nb_src, int *src, int nb_dst, int *ds
       case MODE_RSP:
         explore_rsp(target, nb_vertices, buff, paths, dist); break;
       default:
-        fprintf(stderr,"Warning : graph_molloy_opt::rho() called with Invalid Mode\n");
+        IGRAPH_WARNING("graph_molloy_opt::rho() called with Invalid Mode");
     }
     // remove destinations that weren't discovered by a path coming through
     for(i=0; i<nb_dst; i++) {
@@ -1511,10 +1552,12 @@ double graph_molloy_opt::rho(int mode, int nb_src, int *src, int nb_dst, int *ds
     sum_nij += d*d;
   }
   delete[] times_seen;
-  if(VERBOSE()) {
-    fprintf(stderr,"done\n");
-    if(src_0)  fprintf(stderr,"Warning : %d sources had degree 0\n",src_0);
-    if(nopath) fprintf(stderr,"Warning : %d (src,dst) pairs had no possible path\n",nopath);
+  {
+    igraph_status("done\n", 0);
+    if(src_0)  igraph_warningf("%d sources had degree 0", __FILE__, __LINE__,
+			       -1, src_0);
+    if(nopath) igraph_warningf("%d (src,dst) pairs had no possible path",
+			       __FILE__, __LINE__, -1, nopath);
   }
   return (sum_nij-sum_ni)*double(n)*double(nb_src)/(sum_ni*sum_ni*double(nb_src-1));
 }
@@ -1542,9 +1585,9 @@ int graph_molloy_opt::cycles(int v) {
 return v;
 }
 
-void graph_molloy_opt::remove_vertex(int v) {
-  fprintf(stderr,"Warning : graph_molloy_opt::remove_vertex(%d) called",v);
-}
+// void graph_molloy_opt::remove_vertex(int v) {
+//   fprintf(stderr,"Warning : graph_molloy_opt::remove_vertex(%d) called",v);
+// }
 
 bool graph_molloy_opt::verify(int mode) {
   int i,j,k;
