@@ -28,6 +28,8 @@
 #include <Rinternals.h>
 #include <Rdefines.h>
 
+#include "rinterface.h"
+
 #include <stdlib.h>
 
 /* +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++C */
@@ -205,8 +207,115 @@ SEXP R_igraph_psumtree_draw(SEXP plength, SEXP howmany, SEXP prob) {
   return result;
 }
 
-SEXP R_igraph_srand(SEXP pseed) {
-  unsigned seed=INTEGER(AS_INTEGER(pseed))[0];
-  srand(seed);
+SEXP R_igraph_get_all_simple_paths_pp(SEXP vector) {
+  SEXP result;
+  int i, no=0, n=GET_LENGTH(vector);
+  int *vec=INTEGER(vector), *p=vec, *pp=vec;
+  for (i=0; i<n; i++) {
+    if (vec[i] == 0) { no++; }
+  }
+
+  PROTECT(result=NEW_LIST(no));
+  for (i=0; i<no; i++) {
+    SEXP svec;
+    while (*p != 0) { p++; }
+    SET_VECTOR_ELT(result, i, svec=NEW_INTEGER(p-pp));
+    memcpy(INTEGER(svec), pp, (p-pp) * sizeof(int));
+    pp = ++p;
+  }
+
+  UNPROTECT(1);
+  return result;
+}
+
+SEXP R_igraph_address(SEXP object) {
+  char s[64];
+  snprintf(s, 64, "%p", object);
+  return ScalarString(mkChar(s));
+}
+
+SEXP R_igraph_make_weak_ref(SEXP key, SEXP value, SEXP finalizer) {
+  return R_MakeWeakRef(key, value, finalizer, isNull(finalizer) ? 0 : 1);
+}
+
+SEXP R_igraph_weak_ref_key(SEXP ref) {
+  return R_WeakRefKey(ref);
+}
+
+SEXP R_igraph_weak_ref_value(SEXP ref) {
+  return R_WeakRefValue(ref);
+}
+
+SEXP R_igraph_weak_ref_run_finalizer(SEXP ref) {
+  R_RunWeakRefFinalizer(ref);
   return R_NilValue;
 }
+
+SEXP R_igraph_identical_graphs(SEXP g1, SEXP g2) {
+  int i;
+  for (i = 0; i < 9 ; i++) {
+    if (!R_compute_identical(VECTOR_ELT(g1, i), VECTOR_ELT(g2, i), 0)) {
+      return ScalarLogical(0);
+    }
+  }
+  return ScalarLogical(1);
+}
+
+SEXP R_igraph_graph_version(SEXP graph) {
+  if (GET_LENGTH(graph) == 10 && isEnvironment(VECTOR_ELT(graph, 9))) {
+    SEXP ver = findVar(install(R_IGRAPH_VERSION_VAR), VECTOR_ELT(graph, 9));
+    if (ver != R_UnboundValue) {
+      return ver;
+    } else {
+      return mkString("0.7.999");
+    }
+  } else {
+    return mkString("0.4.0");
+  }
+}
+
+SEXP R_igraph_add_version_to_env(SEXP graph) {
+  uuid_t my_id;
+  char my_id_chr[40];
+
+  PROTECT(graph = duplicate(graph));
+
+  uuid_generate(my_id);
+  uuid_unparse_lower(my_id, my_id_chr);
+  defineVar(install("myid"), mkString(my_id_chr), VECTOR_ELT(graph, 9));
+  defineVar(install(R_IGRAPH_VERSION_VAR), mkString(R_IGRAPH_TYPE_VERSION),
+	    VECTOR_ELT(graph, 9));
+
+  UNPROTECT(1);
+  return graph;
+}
+
+SEXP R_igraph_add_env(SEXP graph) {
+  SEXP result = graph;
+  int i;
+  uuid_t my_id;
+  char my_id_chr[40];
+
+  if (GET_LENGTH(result) != 10) {
+    PROTECT(result = NEW_LIST(10));
+    for (i = 0; i < 9; i++) {
+      SET_VECTOR_ELT(result, i, duplicate(VECTOR_ELT(graph, i)));
+    }
+    SET_ATTRIB(result, duplicate(ATTRIB(graph)));
+    SET_CLASS(result, duplicate(GET_CLASS(graph)));
+  }
+  SET_VECTOR_ELT(result, 9, allocSExp(ENVSXP));
+
+  uuid_generate(my_id);
+  uuid_unparse_lower(my_id, my_id_chr);
+  defineVar(install("myid"), mkString(my_id_chr), VECTOR_ELT(result, 9));
+  defineVar(install(R_IGRAPH_VERSION_VAR), mkString(R_IGRAPH_TYPE_VERSION),
+	    VECTOR_ELT(result, 9));
+
+  if (result != graph) { UNPROTECT(1); }
+  return result;
+}
+
+SEXP R_igraph_get_graph_id(SEXP graph) {
+  return findVar(install("myid"), VECTOR_ELT(graph, 9));
+}
